/* GtkZephyr - a GTK+ based front end to MIT's Zephyr Notification Service
 * Copyright (C) 2000 Sean Dilda
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <zephyr/zephyr.h>
#include <gtk/gtk.h>
#include <time.h>
#include "gtkzephyr.h"
#include "chatdialog.h"
#include "zcalls.h"
#include "errorbox.h"



void gtkzephyr_chat_handle_servack(ZNotice_t *notice, gtkzephyr_subscrip *subscrip) {

  struct tm *senttime = localtime(&(notice->z_time.tv_sec));
  gchar *line, *header, *message;
  gchar **lines;
  gint position, i;

  if (notice->z_kind == SERVNAK) {
    // Not sure what to do here
  } else if (!strcmp(notice->z_message, ZSRVACK_SENT)) {
    /* Do nothing special.. message always gets displayed anyways. */
  } else if (!strcmp(notice->z_message, ZSRVACK_NOTSENT)) {

    gchar *msg = "*** ERROR: User not logged in - following message NOT sent **\n";
    gtk_editable_set_position(GTK_EDITABLE(subscrip->chattext),
	gtk_text_get_length(GTK_TEXT(subscrip->chattext)));
    position = gtk_editable_get_position(GTK_EDITABLE(subscrip->chattext));
    gtk_editable_insert_text(GTK_EDITABLE(subscrip->chattext), msg, strlen(msg),
	&position);

  } else {

    gtkzephyr_error("Unknown server response");
    gtkzephyr_handle_unknown_notice(notice);

  }
  
  if (subscrip->type == GTKZ_USER_TYPE) {
    header = g_strdup_printf("[%.2d:%.2d] <%s> ", senttime->tm_hour,
      	senttime->tm_min, gtkzephyr_get_local_userid());
    message = gtk_editable_get_chars(GTK_EDITABLE(subscrip->chatsendentry), 0, -1);
    gtk_text_freeze(GTK_TEXT(subscrip->chattext));
    gtk_editable_set_position(GTK_EDITABLE(subscrip->chattext),
	gtk_text_get_length(GTK_TEXT(subscrip->chattext)));
    position = gtk_editable_get_position(GTK_EDITABLE(subscrip->chattext));
    if (strlen(message) > 0) {
      lines = g_strsplit(message, "\n", 0);
      for (i = 0; lines[i] != NULL; ++i) {
	line = g_strdup_printf("%s %s\n", header, lines[i]);
	gtk_editable_insert_text(GTK_EDITABLE(subscrip->chattext), line,
	    strlen(line), &position);
	g_free(line);
	if (i == 0) {
	  gchar *tmp = g_strnfill(strlen(header), ' ');
	  g_free(header);
	  header = tmp;
	}
      }
      g_strfreev(lines);
    } else {
      gchar *newheader = g_strdup_printf("%s\n", header);
      gtk_editable_insert_text(GTK_EDITABLE(subscrip->chattext), newheader,
	  strlen(newheader), &position);
      g_free(newheader);
    }
    gtk_text_thaw(GTK_TEXT(subscrip->chattext));
    gtk_adjustment_set_value(GTK_TEXT(subscrip->chattext)->vadj,
	GTK_TEXT(subscrip->chattext)->vadj->upper);
    g_free(header);
    g_free(message);
  }

  gtk_editable_delete_text(GTK_EDITABLE(subscrip->chatsendentry), 0, -1);
  gtk_widget_set_sensitive(subscrip->chatsendentry, TRUE);
  gtk_widget_set_sensitive(subscrip->chatsendbutton, TRUE);
  gtk_widget_grab_focus(subscrip->chatsendentry);

}



void gtkzephyr_chat_handle_message(ZNotice_t *notice, gtkzephyr_subscrip *subscrip) {

  struct tm *senttime = localtime(&(notice->z_time.tv_sec));
  gchar *line, *header, *message, *userid;
  gchar **lines;
  gint position, i;

  userid = gtkzephyr_get_userid(notice->z_sender);
  header = g_strdup_printf("[%.2d:%.2d] <%s> ", senttime->tm_hour,
      senttime->tm_min, userid);
  message = gtkzephyr_get_body(notice->z_message, notice->z_message_len);
  gtk_text_freeze(GTK_TEXT(subscrip->chattext));
  gtk_editable_set_position(GTK_EDITABLE(subscrip->chattext),
      gtk_text_get_length(GTK_TEXT(subscrip->chattext)));
  position = gtk_editable_get_position(GTK_EDITABLE(subscrip->chattext));
  if (message != NULL) {
    lines = g_strsplit(message, "\n", 0);
    for (i = 0; lines[i] != NULL; ++i) {
      line = g_strdup_printf("%s %s\n", header, lines[i]);
      gtk_editable_insert_text(GTK_EDITABLE(subscrip->chattext), line,
	  strlen(line), &position);
      g_free(line);
      if (i == 0) {
	gchar *tmp = g_strnfill(strlen(header), ' ');
	g_free(header);
	header = tmp;
      }
    }
    g_strfreev(lines);
  } else {
    gchar *newheader = g_strdup_printf("%s\n", header);
    gtk_editable_insert_text(GTK_EDITABLE(subscrip->chattext), newheader,
	strlen(newheader), &position);
    g_free(newheader);
  }
  gtk_text_thaw(GTK_TEXT(subscrip->chattext));
  gtk_adjustment_set_value(GTK_TEXT(subscrip->chattext)->vadj,
      GTK_TEXT(subscrip->chattext)->vadj->upper);
  g_free(userid);
  g_free(header);
  g_free(message);
  ZFreeNotice(notice);

}



void close_chat_window(GtkObject *widget, gtkzephyr_subscrip *subscrip) {

  gtk_widget_hide(subscrip->chatwindow);
  gtk_editable_delete_text(GTK_EDITABLE(subscrip->chattext), 0, -1);
  gtk_editable_delete_text(GTK_EDITABLE(subscrip->chatsendentry), 0, -1);
  gtk_widget_set_sensitive(subscrip->chatsendentry, TRUE);
  gtk_widget_set_sensitive(subscrip->chatsendbutton, TRUE);

}



gboolean chatwindow_delete_event(GtkObject *widget, GdkEvent *event, gtkzephyr_subscrip *subscrip) {

  close_chat_window(NULL, subscrip);

  return TRUE;

}



void chat_send_text(GtkEditable *entry, gpointer data) {

  gtkzephyr_subscrip *subscrip = GTKZ_SUBSCRIP(data);
  gtk_widget_set_sensitive(GTK_WIDGET(entry), FALSE);
  gtk_widget_set_sensitive(subscrip->chatsendbutton, FALSE);

  gtkzephyr_send_text(subscrip, entry, gtkzephyr_chat_handle_servack);

}



void make_chat_window(gtkzephyr_subscrip *subscrip) {

  GtkWidget *vbox, *hbuttonbox, *scrolledwindow;

  g_assert(subscrip->chatwindow == NULL);
  g_assert(subscrip->chattext == NULL);
  g_assert(subscrip->chatsendentry == NULL);
  g_assert(subscrip->chatsendbutton == NULL);

  DEBUG_INFO("Chat Window opened for: %s\n", subscrip->type == GTKZ_USER_TYPE ? GTKZ_USER(subscrip)->userid : GTKZ_INST(subscrip)->class);

  subscrip->chatwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize(subscrip->chatwindow, 507, 260);
  gtk_signal_connect(GTK_OBJECT(subscrip->chatwindow), "delete_event",
      GTK_SIGNAL_FUNC(chatwindow_delete_event), subscrip);
  if (subscrip->type == GTKZ_USER_TYPE) {
    gtk_window_set_title(GTK_WINDOW(subscrip->chatwindow),
	GTKZ_USER(subscrip)->userid);
  } else {
    gtk_window_set_title(GTK_WINDOW(subscrip->chatwindow),
	GTKZ_INST(subscrip)->key);
  }
  gtk_container_set_border_width(GTK_CONTAINER(subscrip->chatwindow), 5);
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(subscrip->chatwindow), vbox);
  gtk_widget_show(vbox);
  subscrip->chattext = gtk_text_new(NULL, NULL);
  gtk_text_set_editable(GTK_TEXT(subscrip->chattext), FALSE);
	gtk_text_set_word_wrap(GTK_TEXT(subscrip->chattext), TRUE);
  gtk_widget_show(subscrip->chattext);
  scrolledwindow = gtk_scrolled_window_new(GTK_TEXT(subscrip->chattext)->hadj,
      GTK_TEXT(subscrip->chattext)->vadj);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), subscrip->chattext);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 0);
  gtk_widget_show(scrolledwindow);
  subscrip->chatsendentry = gtk_entry_new();
  gtk_editable_set_editable(GTK_EDITABLE(subscrip->chatsendentry), TRUE);
  gtk_signal_connect(GTK_OBJECT(subscrip->chatsendentry), "activate",
      GTK_SIGNAL_FUNC(chat_send_text), subscrip);
  gtk_box_pack_start(GTK_BOX(vbox), subscrip->chatsendentry, FALSE, FALSE, 3);
  gtk_widget_show(subscrip->chatsendentry);
  hbuttonbox = gtk_hbutton_box_new();
  gtk_button_box_set_child_size(GTK_BUTTON_BOX(hbuttonbox), 65, 15);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbuttonbox), 25);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);
  subscrip->chatsendbutton = gtk_button_new_with_label("Close");
  gtk_signal_connect(GTK_OBJECT(subscrip->chatsendbutton), "clicked",
      GTK_SIGNAL_FUNC(close_chat_window), subscrip);
  gtk_widget_show(subscrip->chatsendbutton);
  gtk_widget_show(hbuttonbox);
  gtk_box_pack_end(GTK_BOX(hbuttonbox), subscrip->chatsendbutton, FALSE, TRUE, 25);
  gtk_box_pack_start(GTK_BOX(vbox), hbuttonbox, FALSE, FALSE, 5);
}



void chat_remove_unreadmsg(ZNotice_t *notice, gtkzephyr_subscrip *subscrip) {

  gtkzephyr_remove_unreadmsg(subscrip);

}



void gtkzephyr_chat_window(gpointer data) {

  gtkzephyr_subscrip *subscrip = GTKZ_SUBSCRIP(data);

  if (subscrip->chatwindow == NULL)
    make_chat_window(subscrip);

  /* If the userdialog is open, it thinks it has full access to all unread
   * messages, so we don't mess with them here */
  if (subscrip->unreadmsg && (!subscrip->userdialog ||
	!GTK_WIDGET_VISIBLE(subscrip->userdialog))) {
    g_list_foreach(subscrip->newmsgs, (GFunc)gtkzephyr_chat_handle_message, subscrip);
    g_list_foreach(subscrip->newmsgs, (GFunc)chat_remove_unreadmsg, subscrip);
    g_list_free(subscrip->newmsgs);
    subscrip->newmsgs = NULL;
  }

  gtk_widget_show(subscrip->chatwindow);
  gdk_window_raise(subscrip->chatwindow->window);

}


void gtkzephyr_open_chat_window(GtkWidget *widget, gpointer data) {

  gtkzephyr_chat_window(data);

}
