/* GtkZephyr - a GTK+ based front end to MIT's Zephyr Notification Service
 * Copyright (C) 2000 Sean Dilda
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <zephyr/zephyr.h>
#include <stdlib.h>
#include "prefs.h"


#define DEFAULT_UPDATE_USERLIST_TIMEOUT_LENGTH 30000 /* 30 seconds */
#define DEFAULT_REDRAW_USERLIST_TIMEOUT_LENGTH 1000 /* 1 second */
#define DEFAULT_MAX_REDRAW_COUNT 20
#define DEFAULT_SHOW_DISCONNECTED FALSE
#define DEFAULT_ADD_NEW_USERS TRUE  /* if someone sends you a personal message and are not
				                                in your .anyone, add them */
// #define CHECK_MESSAGES_TIMEOUT_LENGTH 1000 /* 1 second */
#define DEFAULT_CHECK_MESSAGES_TIMEOUT_LENGTH 10


gtkzephyr_prefstruct *prefstruct;


void gtkzephyr_load_prefs() {

  gchar *value;

  prefstruct = g_new(gtkzephyr_prefstruct, 1);

  value = ZGetVariable("gtkzephyr-update-userlist-timeout");
  if (value)
    prefstruct->update_ul_timeout = strtoul(value, NULL, 10);
  else
    prefstruct->update_ul_timeout = DEFAULT_UPDATE_USERLIST_TIMEOUT_LENGTH;

  value = ZGetVariable("gtkzephyr-redraw-userlist-timeout");
  if (value)
    prefstruct->redraw_ul_timeout = strtoul(value, NULL, 10);
  else
    prefstruct->redraw_ul_timeout = DEFAULT_REDRAW_USERLIST_TIMEOUT_LENGTH;

  value = ZGetVariable("gtkzephyr-check-messages-timeout");
  if (value)
    prefstruct->check_messages_timeout = strtoul(value, NULL, 10);
  else
    prefstruct->check_messages_timeout = DEFAULT_CHECK_MESSAGES_TIMEOUT_LENGTH;

  value = ZGetVariable("gtkzephyr-max-redraw-count");
  if (value)
    prefstruct->max_redraw = strtol(value, NULL, 10);
  else
    prefstruct->max_redraw = DEFAULT_MAX_REDRAW_COUNT;

  value = ZGetVariable("gtkzephyr-show-disconnected");
  if (value) {
    if (!g_strcasecmp(value, "true")) {
      prefstruct->show_disconnected = TRUE;
    } else if (!g_strcasecmp(value, "false")) {
      prefstruct->show_disconnected = FALSE;
    } else {
      prefstruct->show_disconnected = DEFAULT_SHOW_DISCONNECTED;
    }
  } else {
    prefstruct->show_disconnected = DEFAULT_SHOW_DISCONNECTED;
  }

  value = ZGetVariable("gtkzephyr-add-new-users");
  if (value) {
    if (!g_strcasecmp(value, "true")) {
      prefstruct->add_new_users = TRUE;
    } else if (!g_strcasecmp(value, "false")) {
      prefstruct->add_new_users = FALSE;
    } else {
      prefstruct->add_new_users = DEFAULT_ADD_NEW_USERS;
    }
  } else {
    prefstruct->add_new_users = DEFAULT_ADD_NEW_USERS;
  }

}




void gtkzephyr_save_prefs() {

  gchar *tmp_str;

  tmp_str = g_strdup_printf("%d", prefstruct->update_ul_timeout);
  if (prefstruct->update_ul_timeout ==  DEFAULT_UPDATE_USERLIST_TIMEOUT_LENGTH) {
    ZUnsetVariable("gtkzephyr-update-userlist-timeout");
  } else {
    ZSetVariable("gtkzephyr-update-userlist-timeout", tmp_str);
  }
  g_free(tmp_str);

  tmp_str = g_strdup_printf("%d", prefstruct->redraw_ul_timeout);
  if (prefstruct->redraw_ul_timeout == DEFAULT_REDRAW_USERLIST_TIMEOUT_LENGTH) {
    ZUnsetVariable("gtkzephyr-redraw-userlist-timeout");
  } else {
    ZSetVariable("gtkzephyr-redraw-userlist-timeout", tmp_str);
  }
  g_free(tmp_str);

  tmp_str = g_strdup_printf("%d", prefstruct->check_messages_timeout);
  if (prefstruct->check_messages_timeout ==
      DEFAULT_CHECK_MESSAGES_TIMEOUT_LENGTH) {
    ZUnsetVariable("gtkzephyr-check-messages-timeout");
  } else {
    ZSetVariable("gtkzephyr-check-messages-timeout", tmp_str);
  }
  g_free(tmp_str);

  tmp_str = g_strdup_printf("%d", prefstruct->max_redraw);
  if (prefstruct->max_redraw == DEFAULT_MAX_REDRAW_COUNT) {
    ZUnsetVariable("gtkzephyr-max-redraw-count");
  } else {
    ZSetVariable("gtkzephyr-max-redraw-count", tmp_str);
  }
  g_free(tmp_str);

  tmp_str = g_strdup( prefstruct->show_disconnected ? "TRUE" : "FALSE" );
  if (prefstruct->show_disconnected == DEFAULT_SHOW_DISCONNECTED) {
    ZUnsetVariable("gtkzephyr-show-disconnected");
  } else {
    ZSetVariable("gtkzephyr-show-disconnected", tmp_str);
  }
  g_free(tmp_str);

  tmp_str = g_strdup( prefstruct->add_new_users ? "TRUE" : "FALSE" );
  if (prefstruct->add_new_users == DEFAULT_ADD_NEW_USERS) {
    ZUnsetVariable("gtkzephyr-add-new-users");
  } else {
    ZSetVariable("gtkzephyr-add-new-users", tmp_str);
  }
  g_free(tmp_str);

}
