/* GtkZephyr - a GTK+ based front end to MIT's Zephyr Notification Service
 * Copyright (C) 2000 Sean Dilda
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <zephyr/zephyr.h>
#include <gtk/gtk.h>
#include "gtkzephyr.h"
#include "userdialog.h"
#include "zcalls.h"
#include "errorbox.h"


void close_user_dialog(GtkObject *widget, gtkzephyr_subscrip *subscrip) {

  gtk_widget_hide(subscrip->userdialog);
  gtk_editable_delete_text(GTK_EDITABLE(gtk_object_get_data(
      GTK_OBJECT(subscrip->newmsgclist), "text-box")), 0, -1);
  gtk_clist_clear(GTK_CLIST(subscrip->newmsgclist));
  g_list_foreach(subscrip->readmsgs, (GFunc) ZFreeNotice, NULL);
  g_list_free(subscrip->readmsgs);
  gtk_editable_delete_text(GTK_EDITABLE(subscrip->dialogsendtext), 0, -1);
  subscrip->readmsgs = NULL;

}



void reply_callback(GtkObject *object, gtkzephyr_subscrip *subscrip) {

  gtk_notebook_set_page(GTK_NOTEBOOK(subscrip->notebook), 1);

}



gboolean delete_event(GtkObject *widget, GdkEvent *event, gtkzephyr_subscrip *subscrip) {

  close_user_dialog(NULL, subscrip);

  return TRUE;

}



void dialog_send_text(GtkWidget *widget, gpointer data) {

  gtkzephyr_subscrip *subscrip = GTKZ_SUBSCRIP(data);
  gtk_widget_set_sensitive(GTK_WIDGET(subscrip->dialogsendtext), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(subscrip->dialogsendbutton), FALSE);

  gtkzephyr_send_text(subscrip, GTK_EDITABLE(subscrip->dialogsendtext),
      gtkzephyr_user_dialog_handle_servack);

}



void newmsgclist_row_selected(GtkCList *clist, gint row, gint column, GdkEventButton *event, gpointer user_data) {

  GtkText *text = (GtkText *) gtk_object_get_data(GTK_OBJECT(clist),
      "text-box");
  gchar *buffer, *celltext;
  gint tmp = 0;
  ZNotice_t *notice = (ZNotice_t *) gtk_clist_get_row_data(clist, row);

  buffer = gtkzephyr_get_body(notice->z_message, notice->z_message_len);
  gtk_text_freeze(text);
  gtk_editable_delete_text(GTK_EDITABLE(text), 0, -1);
  if (buffer)
    gtk_editable_insert_text(GTK_EDITABLE(text), buffer, strlen(buffer), &tmp);
  gtk_text_thaw(text);
  gtk_clist_get_text(clist, row, 0, &celltext);
  if (celltext[0] == '*') {
    gtkzephyr_subscrip *subscrip = GTKZ_SUBSCRIP(
      	gtk_object_get_data(GTK_OBJECT(clist), "user-struct"));
    gtk_clist_set_text(clist, row, 0, "");
    subscrip->newmsgs = g_list_remove(subscrip->newmsgs, notice);
    subscrip->readmsgs = g_list_append(subscrip->readmsgs, notice);
    gtkzephyr_remove_unreadmsg(subscrip);
  }
  g_free(buffer);
}



void add_to_clist(ZNotice_t *notice, gtkzephyr_subscrip *subscrip) {

  gchar* text[3];
  gint rownumber;
  time_t sec = notice->z_time.tv_sec;

  gtk_clist_freeze(GTK_CLIST(subscrip->newmsgclist));
  DEBUG_INFO("Initializing text..\n");
  text[0] = (gchar *) &"*";
  text[1] = gtkzephyr_get_userid(notice->z_sender);
  text[2] = ctime(&sec);
  text[2][24] = '\0';
  DEBUG_INFO("Appending..\n");
  rownumber = gtk_clist_append(GTK_CLIST(subscrip->newmsgclist), text);
  DEBUG_INFO("Setting row_data..\n");
  gtk_clist_set_row_data(GTK_CLIST(subscrip->newmsgclist), rownumber, notice);
  DEBUG_INFO("Thawing..\n");
  gtk_clist_thaw(GTK_CLIST(subscrip->newmsgclist));
  DEBUG_INFO("Thawed\n");

  g_free(text[1]);

}




void make_user_dialog(gtkzephyr_subscrip *subscrip) {

  GtkWidget *vbox, *tab_label;
  GtkWidget *button, *hbuttonbox, *paned, *scrolledwindow;
  gchar *clisttitles[3] = {"N", "Sender", "Time"};

  g_assert(subscrip->userdialog == NULL);
  g_assert(subscrip->notebook == NULL);
  g_assert(subscrip->newmsgclist == NULL);
  g_assert(subscrip->newmsgtext == NULL);
  g_assert(subscrip->dialogsendtext == NULL);
  g_assert(subscrip->dialogsendbutton == NULL);

  DEBUG_INFO("Dialog opened for: %s\n", subscrip->type == GTKZ_USER_TYPE ? GTKZ_USER(subscrip)->userid : GTKZ_INST(subscrip)->class);

  subscrip->userdialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize(subscrip->userdialog, 300, 225);
  gtk_signal_connect(GTK_OBJECT(subscrip->userdialog), "delete_event",
      GTK_SIGNAL_FUNC(delete_event), subscrip);
  if (subscrip->type == GTKZ_USER_TYPE) {
    gtk_window_set_title(GTK_WINDOW(subscrip->userdialog),
	GTKZ_USER(subscrip)->userid);
  } else {
    gtk_window_set_title(GTK_WINDOW(subscrip->userdialog),
	GTKZ_INST(subscrip)->key);
  }
  gtk_container_set_border_width(GTK_CONTAINER(subscrip->userdialog), 5);
  subscrip->notebook = gtk_notebook_new();
  gtk_container_add(GTK_CONTAINER(subscrip->userdialog), subscrip->notebook);
  gtk_notebook_set_page(GTK_NOTEBOOK(subscrip->notebook), GTK_POS_TOP);
  gtk_widget_show(subscrip->notebook);

  /* read message tab */

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  paned = gtk_vpaned_new();
  subscrip->newmsgclist = gtk_clist_new_with_titles(3, clisttitles);
  gtk_signal_connect(GTK_OBJECT(subscrip->newmsgclist), "select-row",
      GTK_SIGNAL_FUNC(newmsgclist_row_selected), NULL);
  gtk_object_set_data(GTK_OBJECT(subscrip->newmsgclist), "user-struct", subscrip);
  gtk_widget_show(subscrip->newmsgclist);
  scrolledwindow = gtk_scrolled_window_new(
      GTK_CLIST(subscrip->newmsgclist)->hadjustment,
      GTK_CLIST(subscrip->newmsgclist)->vadjustment);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), subscrip->newmsgclist);
  gtk_widget_show(scrolledwindow);
  gtk_paned_add1(GTK_PANED(paned), scrolledwindow);
  subscrip->newmsgtext = gtk_text_new(NULL, NULL);
  gtk_text_set_word_wrap(GTK_TEXT(subscrip->newmsgtext), TRUE);
  gtk_widget_show(subscrip->newmsgtext);
  gtk_object_set_data(GTK_OBJECT(subscrip->newmsgclist), "text-box",
      subscrip->newmsgtext);
  scrolledwindow = gtk_scrolled_window_new(GTK_TEXT(subscrip->newmsgtext)->hadj,
      GTK_TEXT(subscrip->newmsgtext)->vadj);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), subscrip->newmsgtext);
  gtk_widget_show(scrolledwindow);
  gtk_paned_add2(GTK_PANED(paned), scrolledwindow);
  gtk_widget_show(paned);
  gtk_box_pack_start(GTK_BOX(vbox), paned, TRUE, TRUE, 5);
  hbuttonbox = gtk_hbutton_box_new();
  gtk_button_box_set_child_size(GTK_BUTTON_BOX(hbuttonbox), 65, 15);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbuttonbox), 25);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);
  button = gtk_button_new_with_label("Reply");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
      GTK_SIGNAL_FUNC(reply_callback), subscrip);
  gtk_widget_show(button);
  gtk_widget_show(hbuttonbox);
  gtk_box_pack_end(GTK_BOX(hbuttonbox), button, FALSE, TRUE, 25);
  button = gtk_button_new_with_label("Close");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
      GTK_SIGNAL_FUNC(close_user_dialog), subscrip);
  gtk_widget_show(button);
  gtk_widget_show(hbuttonbox);
  gtk_box_pack_end(GTK_BOX(hbuttonbox), button, FALSE, TRUE, 25);
  gtk_box_pack_start(GTK_BOX(vbox), hbuttonbox, FALSE, FALSE, 5);
  gtk_widget_show(vbox);
  tab_label = gtk_label_new("Read Message");
  gtk_widget_show(tab_label);
  gtk_notebook_append_page(GTK_NOTEBOOK(subscrip->notebook), vbox, tab_label);

  /* write message tab */
  
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  gtk_widget_show(vbox);
  subscrip->dialogsendtext = gtk_text_new(NULL, NULL);
  gtk_text_set_editable(GTK_TEXT(subscrip->dialogsendtext), TRUE);
  gtk_widget_show(subscrip->dialogsendtext);
  scrolledwindow = gtk_scrolled_window_new(GTK_TEXT(subscrip->dialogsendtext)->hadj,
      GTK_TEXT(subscrip->dialogsendtext)->vadj);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolledwindow), subscrip->dialogsendtext);
  gtk_widget_show(scrolledwindow);
  gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 0);
  hbuttonbox = gtk_hbutton_box_new();
  gtk_button_box_set_child_size(GTK_BUTTON_BOX(hbuttonbox), 65, 15);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbuttonbox), 25);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);
  gtk_box_pack_start(GTK_BOX(vbox), hbuttonbox, FALSE, FALSE, 0);
  subscrip->dialogsendbutton = gtk_button_new_with_label("Send");
  gtk_signal_connect(GTK_OBJECT(subscrip->dialogsendbutton), "clicked",
      GTK_SIGNAL_FUNC(dialog_send_text), subscrip);
  gtk_widget_show(subscrip->dialogsendbutton);
  gtk_box_pack_end(GTK_BOX(hbuttonbox), subscrip->dialogsendbutton, FALSE, FALSE, 5);
  button = gtk_button_new_with_label("Cancel");
  gtk_signal_connect(GTK_OBJECT(button), "clicked",
      GTK_SIGNAL_FUNC(close_user_dialog), subscrip);
  gtk_widget_show(button);
  gtk_box_pack_end(GTK_BOX(hbuttonbox), button, FALSE, FALSE, 5);
  gtk_widget_show(hbuttonbox);
  tab_label = gtk_label_new("Write Message");
  gtk_widget_show(tab_label);
  gtk_notebook_append_page(GTK_NOTEBOOK(subscrip->notebook), vbox, tab_label);
}



void open_user_dialog(gtkzephyr_subscrip *subscrip) {

  if (subscrip->notebook == NULL)
    make_user_dialog(subscrip);

  if (subscrip->unreadmsg && !GTK_WIDGET_VISIBLE(subscrip->userdialog)) {
    g_list_foreach(subscrip->newmsgs, (GFunc)add_to_clist, subscrip);
  }

}



void gtkzephyr_open_read(GtkWidget *widget, gpointer data) {

  gtkzephyr_subscrip *subscrip = GTKZ_SUBSCRIP(data);

  open_user_dialog(subscrip);
  gtk_notebook_set_page(GTK_NOTEBOOK(subscrip->notebook), 0);
  if (subscrip->unreadmsg && !GTK_WIDGET_VISIBLE(subscrip->userdialog)) {
    gtk_clist_select_row(GTK_CLIST(subscrip->newmsgclist), 0, 0);
  }
  gtk_widget_show(GTK_WIDGET(subscrip->userdialog));
  gdk_window_raise(subscrip->userdialog->window);

}



void gtkzephyr_open_write(GtkWidget *widget, gpointer data) {

  gtkzephyr_subscrip *subscrip = GTKZ_SUBSCRIP(data);

  open_user_dialog(subscrip);
  gtk_notebook_set_page(GTK_NOTEBOOK(subscrip->notebook), 1);
  gtk_widget_grab_focus(subscrip->dialogsendtext);
  gtk_widget_show(GTK_WIDGET(subscrip->userdialog));
  gdk_window_raise(subscrip->userdialog->window);

}



void gtkzephyr_open_info(GtkWidget *widget, gpointer data) {

  gtkzephyr_subscrip *subscrip = GTKZ_SUBSCRIP(data);

  open_user_dialog(subscrip);
  gtk_notebook_set_page(GTK_NOTEBOOK(subscrip->notebook), 2);
  gtk_widget_show(GTK_WIDGET(subscrip->userdialog));

}


void gtkzephyr_user_dialog(gtkzephyr_subscrip *subscrip) {

  if (subscrip->unreadmsg)
    gtkzephyr_open_read(NULL, subscrip);
  else
    gtkzephyr_open_write(NULL, subscrip);

}



void gtkzephyr_user_dialog_handle_message(ZNotice_t *notice, gtkzephyr_subscrip *subscrip) {

  
  subscrip->newmsgs = g_list_append(subscrip->newmsgs, notice);

  if (subscrip->userdialog && GTK_WIDGET_VISIBLE(subscrip->userdialog)) {
    DEBUG_INFO("About to add message to clist\n");
    add_to_clist(notice, subscrip);
    DEBUG_INFO("Message added to clist\n");
  }

  DEBUG_INFO("About to call gtkzephyr_add_unreadmsg\n");
  gtkzephyr_add_unreadmsg(subscrip);
  DEBUG_INFO("gtkzephyr_add_unreadmsg call completed.\n");
  
}



void gtkzephyr_user_dialog_handle_servack(ZNotice_t *notice, gtkzephyr_subscrip *subscrip) {
  DEBUG_INFO("Entering gtkzephyr_user_dialog_handle_servack\n");
  if (notice->z_kind == SERVNAK) {
    // Don't know what to do here
  } else { /* z_kind == SERVACK */
    if (!strcmp(notice->z_message, ZSRVACK_SENT)) {
      /* Do nothing special.. (closing the window is always done) */
    } else if (!strcmp(notice->z_message, ZSRVACK_NOTSENT)) {

      // FIXME Make this able to handle instances as well
      gchar *error = g_strdup_printf("%s is not logged in", notice->z_recipient);
      gtkzephyr_error(error);
      g_free(error);

    } else {

      gtkzephyr_error("Unknown server response");
      gtkzephyr_handle_unknown_notice(notice);

    }
  }

  close_user_dialog(NULL, subscrip);
  gtk_editable_delete_text(GTK_EDITABLE(subscrip->dialogsendtext), 0, -1);
  gtk_widget_set_sensitive(GTK_WIDGET(subscrip->dialogsendtext), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(subscrip->dialogsendbutton), TRUE);

}
